"""File renaming utilities."""
import os
import urllib
import re
    
def getExtension(filename):
    """Returns the file extension."""
    parts = filename.split(".")
    if len(parts) > 1:
        return parts[-1]
    else:
        return ""        
        
def removeExtension(filename):
    """Returns filename with the extension removed."""
    parts = filename.split(".")
    if len(parts) > 1:
        return ".".join(parts[:-1])
    else:
        return filename       
        
def rename_prefix(files, s):
    """Renames all files in files prefixing then with the string s.
    
    Arguments:
    files -- a sequence of path strings.
    s -- an string.
    """
    for f in files:      
        (filedir, filename) = os.path.split(f)
        newName = s + filename
        os.rename(f, os.path.join(filedir, newName))
        
def rename_suffix(files, s, preserveExtension = True):
    """Renames all files in files suffixing then with the string s.
    
    Arguments:
    files -- a sequence of path strings.
    s -- an string.
    preserveExtension -- preserve file extension ?
    """
    for f in files:
        (filedir, filename) = os.path.split(f)
            
        if preserveExtension:
            extension = getExtension(filename)
            newName = removeExtension(filename) + s
            if len(extension) > 0:
                newName += "." + extension            
        else:
            newName = filename + s
                                           
        os.rename(f, os.path.join(filedir, newName))        
        
def rename_replace(files, old, new):
    """
    Renames all files in files replacing the string old by new.
    
    Arguments:
    files -- a sequence of path strings.
    old -- an string.
    new -- an string.
    """
    for f in files:
        (filedir, filename) = os.path.split(f)
        newName = filename.replace(old, new)
        os.rename(f, os.path.join(filedir, newName))    

def rename_delete_first_chars(files, n):
    """
    Renames all files in files deleting the n first chars.
    
    Arguments:
    files -- a sequence of path strings.
    n -- the number of chars to delete.
    """
    for f in files:
        (filedir, filename) = os.path.split(f)
        newName = filename[n:]
        os.rename(f, os.path.join(filedir, newName))  

def fixTitleCase(fileName):
    """Fix the string.title() issue with '. E.g.: takes "You Don'T Know" and 
    returns "You Don't Know".
    """
    
    fileNameParts = fileName.split("'")
    fixedParts = [fileNameParts[0]]
    
    for part in fileNameParts[1:]:
        if part:
            fixedParts.append(part[0].lower() + part[1:])
        else:
            fixedParts.append(part)
                    
    return "'".join(fixedParts)       
    
def fixTrackNumber(fname):
    """
    Replace ' ' by '-' after the track number, if needed.
    
    """
    TRACK_NUMBER_PATTERN = r'(\d\d?)( |-)'
    
    match = re.match(TRACK_NUMBER_PATTERN, fname)
    if not match:
        return fname
    
    matched = match.group()
    track_number = match.group(1)
    separator = '-'
    
    if len(track_number) == 1:
        track_number = '0' + track_number
        
    return fname.replace(matched, track_number + separator)      
               
def unhide(filename):
    if filename[0].startswith('.'):
        os.rename(filename, filename[1:])
    



import os
import sys
import renaming
import os.path
import re

def unhide():
    """
    Usage: unhide FILES
    
    Unhides all hidden files in FILES.
    """
    files = sys.argv

    for f in files:
        renaming.unhide(f)

def rensuf():
    """
    Usage: rensuf SUFFIX FILES
    
    Renames all files in FILES adding SUFFIX in the end of the file name. File 
    extension is preserved.
    """
    s = sys.argv[1]
    files = sys.argv[2:]
    
    renaming.rename_suffix(files, s)            
    
def renrep():
    """
    Usage: renrep OLDSTR NEWSTR FILES
    
    Renames all files in FILES replacing OLDSTR by NEWSTR.
    """
    old = sys.argv[1]
    new = sys.argv[2]
    files = sys.argv[3:]
    
    renaming.rename_replace(files, old, new)
    
        
def renpre():
    """
    Usage: renpre PREFFIX FILES
    
    Renames all files in FILES adding PREFFIX in the beggining of the file name. 
    File extension is preserved.
    """
    s = sys.argv[1]
    files = sys.argv[2:]
    
    renaming.rename_prefix(files, s)        
    

    

def renmp3():
    """
    Usage: renmp3 FILES
    
    Renames all files in FILES performing the following operations:
    - replaces '_' by ' ';
    - replaces ' - ' and '- ' by '-';
    - titlecase the names;
    - lowcase file extension;
    - remove leading and trailing spaces;
    - replace ' ' by '-' after the track number, if needed;
    - makes track numbers have always 2 digits.
    """
    paths = sys.argv[1:]
        
    for path in paths:
        if not path:
            continue
        
        if path[-1] == os.sep:
            path = path[:-1]
        
        (filedir, filename) = os.path.split(path)
        
        if not filename:
            continue
        
        extension = renaming.getExtension(filename)
        newName = renaming.removeExtension(filename)
            
        newName = newName \
            .replace("_", " ") \
            .replace(" - ", "-") \
            .replace("- ", "-") \
            .title() \
            .strip()
        newName = renaming.fixTrackNumber(newName)
        newName = renaming.fixTitleCase(newName)
        
        if extension:
            newName += "." + extension.lower()        
        
        newPath = os.path.join(filedir, newName)
                           
        os.rename(path, newPath)
        
                
def renlu():
    """
    Usage: renlu FILES
    
    Renames all files in FILES performing the following operations:
    - replaces ' ' by '_';
    - replaces ' - ' and '- ' by '-';
    - lowercase the names;
    - remove leading and trailing spaces.
    """
    files = sys.argv[1:]
        
    for f in files:
        (filedir, filename) = os.path.split(f)
        if len(filename) <= 0:
            continue
        
        extension = renaming.getExtension(filename)
        newName = renaming.removeExtension(filename)
            
        newName = newName.replace(" ", "_").replace(" - ", "-").\
            replace("- ", "-").lower().strip()

        if len(extension) > 0:
            newName += "." + extension.lower()        
                                    
        os.rename(f, os.path.join(filedir, newName))       
        
def rendel():
    """
    Usage: rendel STR FILES
    
    Deletes the string STR from the names of all files in FILES.
    """
    s = sys.argv[1]
    files = sys.argv[2:]
    
    renaming.rename_replace(files, s, "")
                 
                 
def rendeln():
    """
    Usage: rendeln N FILES
    
    Deletes the N first characters from the names of all files in FILES.
    """
    n = int(sys.argv[1])
    files = sys.argv[2:]
    
    renaming.rename_delete_first_chars(files, n)
                 



from setuptools import setup, find_packages
import os

version = '0.2'





setup(name='rbco.rename',
      version=version,
      description="A set of Python scripts to rename files in batch.",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='shell unix filesystem',
      author='Rafael Oliveira',
      author_email='rafaelbco@gmail.com',
      url='',
      license='LGPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['rbco'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points={
          'console_scripts': [
              'renpre = rbco.rename.console:renpre',
              'rensuf = rbco.rename.console:rensuf',
              'unhide = rbco.rename.console:unhide',
              'renlu = rbco.rename.console:renlu',
              'renmp3 = rbco.rename.console:renmp3',
              'renrep = rbco.rename.console:renrep',                    
              'rendeln = rbco.rename.console:rendeln',                    
              'rendel = rbco.rename.console:rendel',      
          ]
      },
)
